(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_hashtbl.mli,v 1.6 2013-07-02 10:20:24 fclement Exp $*)

(** {3 Matrix hash tables}

 Specialized hash tables for numbering matrix slots, i.e. couples of natural
 numbers. In this case, a costless bijective hash function can be defined.
 Type [Umatrix_hashtbl.t] is an efficient alternative to
 [(Unatural.t * Unatural.t, Unatural.t) Hashtbl.t]. *)

type size = Upervasives.length * Upervasives.length;;
type key = Umatrix_index.element;;
type value = Umatrix_index.index;;

type t = private (size, key) Ugen_numbering_hashtbl.t;;

external of_t : t -> (size, key) Ugen_numbering_hashtbl.t = "%identity";;

(** {6 Construction function} *)

val make : size -> t;;

(** {6 Access functions} *)

val length : t -> Upervasives.length;;
val size : t -> size;;

val mem : t -> key -> bool;;

val value_of_key : t -> key -> value;;

val key_of_value : t -> value -> key;;

(** {6 Iterators} *)

val iter : (key -> value -> unit) -> t -> unit;;

val fold_left : ('accu -> key -> value -> 'accu) -> 'accu -> t -> 'accu;;
