(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unonempty_set.mli,v 1.2 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 Nonempty sets} *)

type 'a uindex = 'a Uuniverse.index;;

type 'a index = 'a uindex uindex;;

type 'a t = private
  {
   universe : 'a Uuniverse.t;
   uindices : 'a uindex Uuniverse.t;
  }
;;

(** {6 Construction functions} *)

val singleton : 'a Uuniverse.t -> 'a uindex -> 'a t;;
val full : 'a Uuniverse.t -> 'a t;;

val same_full : 'a t -> 'a t -> bool;;
val check_same_full : Upervasives.function_name -> 'a t -> 'a t -> unit;;

val of_array : 'a Uuniverse.t -> 'a uindex array -> 'a t;;
val of_list : 'a Uuniverse.t -> 'a uindex Ulist1.t -> 'a t;;

val of_rectangle : int -> int -> (int * int) t;;
val of_natural_rectangle :
  Unatural.t -> Unatural.t -> (Unatural.t * Unatural.t) t
;;

(** {6 Modification functions} *)

val add : 'a t -> 'a uindex -> unit;;
val union : 'a t -> 'a t -> 'a t;;

(** {6 Indices and elements} *)

val cardinal : 'a t -> Unatural.t;;

val valid_index : 'a t -> 'a index -> bool;;
val check_index : Upervasives.function_name -> 'a t -> 'a index -> unit;;

val mem : 'a t -> 'a uindex -> bool;;

val choice : 'a t -> 'a uindex;;

val uindex : 'a t -> 'a index -> 'a uindex;;
val uindices : 'a t -> 'a uindex array;;

val element : 'a t -> 'a index -> 'a;;
val elements : 'a t -> 'a array;;

(** {6 Iterators} *)

val fold_left : ('a -> 'b uindex -> 'a) -> 'a -> 'b t -> 'a;;
val fold_left_i : ('a -> 'b index -> 'b uindex -> 'a) -> 'a -> 'b t -> 'a;;
val iter : ('a uindex -> unit) -> 'a t -> unit;;
val iter_i : ('a index -> 'a uindex -> unit) -> 'a t -> unit;;
val map_indices : ('a uindex -> 'b) -> 'a t -> 'b array;;
val map : ('a uindex -> 'b uindex) -> 'a t -> 'b Uuniverse.t -> 'b t;;

val find : ('a uindex -> bool) -> 'a t -> 'a uindex;;
val find_all : ('a uindex -> bool) -> 'a t -> 'a uindex list;;
val filter : ('a uindex -> bool) -> 'a t -> 'a uindex list;;

(** {6 Nonempty subsets} *)

type 'a nonempty_subset =
  {
   support : 'a t;
   indices : 'a index array;
   nonempty_set : 'a t;
  }
;;

val full_index_of_uindex : 'a uindex -> 'a index;;

val subset_of_full : 'a t -> 'a t -> 'a nonempty_subset;;

type 'a dichotomy = 'a nonempty_subset * 'a nonempty_subset;;

val dichotomy : ('a index -> 'a uindex -> bool) -> 'a t -> 'a dichotomy;;
